//precision highp float;

attribute vec3 a_position;
attribute vec2 a_uv;
attribute vec2 a_depthuv;
varying vec3 v_texcoord;
varying vec4 v_position;
varying vec2 v_depth;

uniform vec4 u_distance_max;
uniform mat4 u_mvp;
uniform sampler2D u_depth_texture;

vec3 getNormal(in vec2 rg) {
    
    float tz = floor(rg.y / 64.); //8 * 8
    float z = ((rg.x - floor(rg.x / 2.) * 2.) * 4.) + tz;
    float y = floor((rg.y - tz * 64.) / 8.);
    float x = floor(mod(rg.y, 8.));
    
    if (x > 3.) { x += -8.; }
    if (y > 3.) { y += -8.; }
    if (z > 3.) { z += -8.; }
    
    return vec3(x, y, z);
}

float getDepth(in vec3 position, in vec2 uv, in float maxValue) {
    if(uv.x > 0.98)
    {
        uv.x = 0.0;
    }
    
    vec4 depthColor = texture2D(u_depth_texture, uv);
    depthColor *= 255.;
    
    float depth = pow(maxValue + 1., depthColor.b / 256.) - 1.;

    if ( depth <= 3.0 ) {
        vec3 normal = getNormal(depthColor.rg);
        if( length(normal) == 0.0 )
//        if( uv.y < 0.6 )
        {
            depth = maxValue;
        }
        else
        {
            depth = 3.0 + 3.0 * (length(position) / abs(position.z) - 1.0);
        }
    }
    
    return depth;
}
                                
void main()
{
    vec3 dPosition = a_position;
    float depth = getDepth(a_position, a_depthuv, u_distance_max.x);
    v_depth.x = depth;
    v_depth.y = u_distance_max.x;
    dPosition *= depth;
    vec3 texcoord = vec3(a_uv.xy, 1.0);
    texcoord *= length(dPosition) / u_distance_max.x;
    v_texcoord = texcoord;
    gl_Position = u_mvp * vec4( dPosition, 1.0 );
}
